import InfoTooltip from "../InfoTooltip.js";

export default {
    props: {
        frame: Object,
        selectedExamples: Object
    },
    components: {InfoTooltip},
    emits: ['frameSelectionChanged'],
    data() {
        return {
            img_prefix: String,
        }
    },
    methods: {
        getTitleHTML() {
            const ret = window.lexical_units2dom(this.frame.lexical_units);
            return ret;
        },
        getOpinionHTML() {
            return window.make_opinion_row(this.frame, this.frame.arguments.length, 3).outerHTML;
        },
        getArguments() {
            return this.frame.arguments;
        },
        selectArgument(argument) {
            argument.selected = !argument.selected;
            const selectedArguments = this.frame.arguments.filter(argument => argument.selected)
            this.$emit('frameSelectionChanged', selectedArguments);
        },
        computeArgumentCSS(argument) {
            const selectedExampleFrameArguments = this.selectedExamples && this.selectedExamples.length > 0 ? new Set(this.selectedExamples.map(e => e.argument_ids).flat()) : null;
            return argument.role + ' ' + (argument.selected ? 'active' : argument.hover ? 'bg-highlight' : '')
            + (selectedExampleFrameArguments != null ? selectedExampleFrameArguments.has(argument.id) ? 'example-yes' : 'example-no' : '');
        }
    },
    mounted () {
        this.img_prefix = window.STATIC_URL;
    },
    template: `
        <table class="table m-0 table-borderless border border-secondary text-dark">
            <tbody>
                <tr class="opinion-row" v-html="getOpinionHTML()">
                </tr>
                <tr>
                    <th scope="row" class="py-2 px-1 text-secondary">Rola</th>
                    <template v-for="argument in frame.arguments">
                        <td class="argument py-2 px-1 border-top border-left border-secondary" 
                            @mouseenter="argument.hover=true"
                            @mouseleave="argument.hover=false"
                            @click.stop="selectArgument(argument)"
                            :class="computeArgumentCSS(argument)" 
                        >
                            {{argument.str}}
                        </td>
                    </template>
                </tr>
                <tr>
                    <th scope="row" class="py-0 px-1 text-secondary">Preferencje selekcyjne</th>
                    <template v-for="argument in getArguments()">
                    
                    <td class="preferences py-0 px-0 border-top border-left border-secondary"
                        :class="argument.selected ? 'active' : argument.hover ? 'bg-highlight' : ''" 
                    >
                        <template v-if="argument.preferences.length > 0" v-for='preference in argument.preferences'>
                            <div
                              v-if="preference.url != null"
                              class="preference py-2 px-1"
                            >
                              <a class="synset-plwn" v-bind:href="preference.url" target="_blank">{{ preference.str }}</a>
                              <info-tooltip v-if="preference.info" :text="'definicja: <i>'+ preference.info +'</i>'" />
                            </div>
                            <div v-else class="preference py-2 px-1">{{ preference.str }}</div>
                        </template>
                        </td>
                    </template>
                </tr>
            </tbody>
        </table>
  `
}


